/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::NJointKinematicUnitPassThroughController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <atomic>

#include <VirtualRobot/Robot.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "NJointController.h"
#include "../RobotUnit.h"
#include "../SensorValues/SensorValue1DoFActuator.h"

#include "../ControlTargets/ControlTarget1DoFActuator.h"

namespace armarx
{
    TYPEDEF_PTRS_HANDLE(NJointKinematicUnitPassThroughControllerConfig);
    class NJointKinematicUnitPassThroughControllerConfig : virtual public NJointControllerConfig
    {
    public:
        std::string deviceName;
        std::string controlMode;
    };


    TYPEDEF_PTRS_HANDLE(NJointKinematicUnitPassThroughController);
    /**
     * @brief The NJointKinematicUnitPassThroughController class
     * @ingroup Library-RobotUnit-NJointControllers
     */
    class NJointKinematicUnitPassThroughController:
        virtual public NJointController
    {
    public:
        using ConfigPtrT = NJointKinematicUnitPassThroughControllerConfigPtr;

        inline NJointKinematicUnitPassThroughController(
            RobotUnitPtr prov,
            const NJointKinematicUnitPassThroughControllerConfigPtr& cfg,
            const VirtualRobot::RobotPtr&);

        inline void rtRun(const IceUtil::Time&, const IceUtil::Time&) override
        {
            *target = control;
        }
        inline void rtPreActivateController() override
        {

        }

        void reset()
        {
            control = (*sensor) * sensorToControlOnActivateFactor;
            if (std::abs(control) < resetZeroThreshold)
            {
                control = 0;
            }
        }

        //ice interface
        inline std::string getClassName(const Ice::Current& = Ice::emptyCurrent) const override
        {
            return "NJointKinematicUnitPassThroughController";
        }

        void set(float val)
        {
            control = val;
        }

    protected:
        std::atomic<float> control {0};
        float* target {nullptr};
        const float* sensor
        {
            nullptr
        };
        float sensorToControlOnActivateFactor {0};
        float resetZeroThreshold {0};
    };
}
