/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::CartesianWaypointControlGuiWidgetController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2019
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <VirtualRobot/Robot.h>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/SimpleConfigDialog/SimpleConfigDialog.h>

#include <RobotAPI/interface/core/RobotState.h>
#include <RobotAPI/interface/units/RobotUnit/RobotUnitInterface.h>
#include <RobotAPI/interface/units/RobotUnit/NJointCartesianWaypointController.h>

#include <RobotAPI/gui-plugins/CartesianWaypointControlGui/ui_CartesianWaypointControlGuiWidget.h>

namespace armarx
{
    /**
    \page RobotAPI-GuiPlugins-CartesianWaypointControlGui CartesianWaypointControlGui
    \brief The CartesianWaypointControlGui allows visualizing ...

    \image html CartesianWaypointControlGui.png
    The user can

    API Documentation \ref CartesianWaypointControlGuiWidgetController

    \see CartesianWaypointControlGuiGuiPlugin
    */

    /**
     * \class CartesianWaypointControlGuiWidgetController
     * \brief CartesianWaypointControlGuiWidgetController brief one line description
     *
     * Detailed description
     */
    class ARMARXCOMPONENT_IMPORT_EXPORT
        CartesianWaypointControlGuiWidgetController:
        public armarx::ArmarXComponentWidgetControllerTemplate < CartesianWaypointControlGuiWidgetController >
    {
        Q_OBJECT

    public:
        /// Controller Constructor
        explicit CartesianWaypointControlGuiWidgetController();

        /// Controller destructor
        virtual ~CartesianWaypointControlGuiWidgetController();

        /// @see ArmarXWidgetController::loadSettings()
        void loadSettings(QSettings* settings) override;

        /// @see ArmarXWidgetController::saveSettings()
        void saveSettings(QSettings* settings) override;

        /**
         * Returns the Widget name displayed in the ArmarXGui to create an
         * instance of this class.
         */
        static QString GetWidgetName()
        {
            return "RobotControl.CartesianWaypointControlGui";
        }

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;

        QPointer<QDialog> getConfigDialog(QWidget* parent) override;
        void configured() override;

    signals:
        /* QT signal declarations */

    private slots:
        void on_pushButtonStop_clicked();
        void on_pushButtonExecute_clicked();
        void on_pushButtonZeroFT_clicked();
        void on_pushButtonSendSettings_clicked();
        void on_pushButtonCreateController_clicked();

        void triggerParsing();

    private:
        NJointCartesianWaypointControllerRuntimeConfig readRunCfg() const;
        void deleteOldController();

    private:
        std::string                                     _robotStateComponentName;
        std::string                                     _robotUnitName;
        RobotStateComponentInterfacePrx                 _robotStateComponent;
        RobotUnitInterfacePrx                           _robotUnit;
        Ui::CartesianWaypointControlGuiWidget           _ui;
        QPointer<SimpleConfigDialog>                    _dialog;
        NJointCartesianWaypointControllerInterfacePrx   _controller;
        std::string                                     _controllerName;
        VirtualRobot::RobotPtr                          _robot;
        std::vector<Eigen::Matrix4f>                    _lastParsedWPs;
        bool                                            _supportsFT{false};
    };
}


