/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::GuiHealthClientWidgetController
 * \author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * \date       2018
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "GuiHealthClientWidgetController.h"

#include <QLabel>
#include <string>
#include <QPushButton>

using namespace armarx;

GuiHealthClientWidgetController::GuiHealthClientWidgetController()
{
    widget.setupUi(getWidget());

    healthTimer = new QTimer(this);
    healthTimer->setInterval(10);
    connect(healthTimer, SIGNAL(timeout()), this, SLOT(healthTimerClb()));
    connect(widget.pushButtonUnregister, SIGNAL(clicked()), this, SLOT(unregisterSelf()));

    updateSummaryTimer = new QTimer(this);
    updateSummaryTimer->setInterval(100);
    connect(updateSummaryTimer, SIGNAL(timeout()), this, SLOT(updateSummaryTimerClb()));
}


GuiHealthClientWidgetController::~GuiHealthClientWidgetController()
{

}


void GuiHealthClientWidgetController::loadSettings(QSettings* settings)
{

}

void GuiHealthClientWidgetController::saveSettings(QSettings* settings)
{

}


void GuiHealthClientWidgetController::onInitComponent()
{
    usingProxy("RobotHealth");

}

void GuiHealthClientWidgetController::healthTimerClb()
{
    RobotHealthHeartbeatArgs rhha;
    rhha.maximumCycleTimeWarningMS = 250;
    rhha.maximumCycleTimeErrorMS = 500;
    robotHealthTopicPrx->heartbeat(getName(), RobotHealthHeartbeatArgs());
}
void GuiHealthClientWidgetController::updateSummaryTimerClb()
{
    if (robotHealthComponentPrx)
    {
        try
        {
            widget.labelHealthState->setText(QString::fromUtf8(robotHealthComponentPrx->getSummary().c_str()));
        }
        catch (...)
        {}
    }
}

void GuiHealthClientWidgetController::unregisterSelf()
{
    healthTimer->stop();
    robotHealthTopicPrx->unregister(getName());
    widget.labelState->setText("inactive.");
    widget.pushButtonUnregister->setDisabled(true);
}


void GuiHealthClientWidgetController::onConnectComponent()
{
    robotHealthTopicPrx = getTopic<RobotHealthInterfacePrx>("RobotHealthTopic");
    robotHealthComponentPrx = getProxy<RobotHealthComponentInterfacePrx>("RobotHealth");
    healthTimer->start();
    updateSummaryTimer->start();
    QLabel* labelState = widget.labelState;
    labelState->setText("Sending heartbeats ...");
}

void GuiHealthClientWidgetController::onDisconnectComponent()
{
    healthTimer->stop();
    updateSummaryTimer->stop();
}
