#include "JointPWMTorqueController.h"
#include "../KITGripperBasisBoard.h"

#include <ArmarXCore/core/logging/Logging.h>

using namespace armarx;


JointPWMTorqueController::JointPWMTorqueController(const std::string deviceName, KITGripperBasisBoardPtr board, ActorDataPtr jointData, MotorTorqueEstimatorPtr torqueEstimation)
    : JointController(), target(), board(board), deviceName(deviceName), torqueEstimation(torqueEstimation)
{
    actorIndex = board->getActorIndex(deviceName);
    dataPtr = jointData;
}

JointPWMTorqueController::~JointPWMTorqueController() noexcept(true)
{
    stopRequested = true;
    try
    {
    }
    catch (...)
    {
    }
}

void JointPWMTorqueController::rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration)
{
    if (target.isValid())
    {
        const float velocity = dataPtr->getRawVelocity(); // dataPtr->getVelocity();
        const float targetTorque = target.torque;

        const int maxIter = 50;
        int counter = 0;

        float pwm_approx_min = -3000.f;
        float pwm_approx_max = 3000.f;
        float pwm_approx_estimate = (pwm_approx_min + pwm_approx_max) * 0.5;

        auto f = [ = ](float pwm)
        {
            int32_t pwm_integer = (int32_t) pwm;
            torqueEstimation->estimateTorque(velocity, pwm_integer);
            return torqueEstimation->getTorqueWithInertia() - targetTorque;
        };

        while (std::fabs(f(pwm_approx_estimate)) > .005f && counter < maxIter)
        {
            if (f(pwm_approx_min) * f(pwm_approx_max) < 0)
            {
                if (f(pwm_approx_estimate) * f(pwm_approx_min) < 0)
                {
                    pwm_approx_max = pwm_approx_estimate;
                }
                else
                {
                    pwm_approx_min = pwm_approx_estimate;
                }
                pwm_approx_estimate = (pwm_approx_max + pwm_approx_min) / 2;
            }
            else
            {
                ARMARX_INFO << deactivateSpam(1) << "ASSERT";
                break;
            }
            counter++;
        }

        /*if (f(pwm_approx1) * f(pwm_approx2) >= 0)
                {
                    //ARMARX_INFO << deactivateSpam(1) << "ASSERT";
                    dataPtr->setTargetPWM(0);
                    return;
                }

                float pwm_approx3 = pwm_approx1;

                for (; counter < maxIter; counter++)
                {
                    pwm_approx3 = (pwm_approx1 * f(pwm_approx2) - pwm_approx2 * f(pwm_approx1)) / (f(pwm_approx2) - f(pwm_approx1));
                    if (abs(f(pwm_approx3)) <= .01)
                    {
                        break;
                    }
                    else if (f(pwm_approx3) * f(pwm_approx1) < 0)
                    {
                        pwm_approx2 = pwm_approx3;
                    }
                    else
                    {
                        pwm_approx1 = pwm_approx3;
                    }
                }*/




        ARMARX_INFO << deactivateSpam(0.01) << VAROUT(targetTorque) << VAROUT(pwm_approx_estimate) << VAROUT(f(pwm_approx_estimate)) << VAROUT(counter);
        dataPtr->setTargetPWM(int(pwm_approx_estimate));
        //dataPtr->setTargetPWM(0);

        //        torqueEstimation->estimateTorque(dataPtr->getVelocity(), dataPtr->getTargetPWM());
        //        float torque_error = targetTorque - torqueEstimation->getTorqueWithInertia();
        //        float P = 150.0f;
        //        float I = 20.0f;
        //        error_sum += torque_error;
        //        float pwm =  P * torque_error + I * error_sum;
        //        pwm = (1 - filter_coeff) * last_pwm + filter_coeff * pwm;
        //        dataPtr->setTargetPWM(int(pwm));
        //        last_pwm = pwm;
    }
}

ControlTargetBase* JointPWMTorqueController::getControlTarget()
{
    return &target;
}

void JointPWMTorqueController::rtPreActivateController()
{
}

void JointPWMTorqueController::rtPostDeactivateController()
{
    ARMARX_RT_LOGF_INFO("Setting PWM to 0");
    dataPtr->setTargetPWM(0);
}

StringVariantBaseMap JointPWMTorqueController::publish(const DebugDrawerInterfacePrx& draw, const DebugObserverInterfacePrx& observer) const
{
    return {};
}





