#pragma once

#include <RobotAPI/components/units/RobotUnit/JointControllers/JointController.h>
#include <RobotAPI/components/units/RobotUnit/ControlTargets/ControlTarget1DoFActuator.h>
#include "../KITGripperBasisBoardData.h"
#include "../Misc/TorqueEstimation.h"


namespace armarx
{
    using KITGripperBasisBoardPtr = std::shared_ptr<class KITGripperBasisBoard>;


    class JointPWMTorqueController;
    typedef std::shared_ptr<JointPWMTorqueController> JointPWMTorqueControllerPtr;

    class JointPWMTorqueController : public JointController
    {
    public:
        JointPWMTorqueController(const std::string deviceName, KITGripperBasisBoardPtr board, ActorDataPtr jointData, MotorTorqueEstimatorPtr torqueEstimation);
        ~JointPWMTorqueController() noexcept(true);

        void rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) override ;
        ControlTargetBase* getControlTarget() override;

        void rtPreActivateController() override;
    protected:
        ControlTarget1DoFActuatorTorque target;
        ActorDataPtr dataPtr;

        KITGripperBasisBoardPtr board;
        const std::string deviceName;
        float last_pwm = 0.0f;
        float filter_coeff = 0.8f;
        float error_sum = 0.0f;
        MotorTorqueEstimatorPtr torqueEstimation;

        size_t actorIndex = 0;
        bool stopRequested = false;
    protected:
        void rtPostDeactivateController() override;
    public:
        StringVariantBaseMap publish(const DebugDrawerInterfacePrx& draw, const DebugObserverInterfacePrx& observer) const override;
    };
}
