#include "TorqueEstimation.h"
#include <math.h>
#include <ArmarXCore/core/logging/Logging.h>

/* --------- !!! Do not change the following function for motor model "motor16" !!! ----------- */
float MotorTorqueEstimator::linearModel_dcx16(int32_t nI, int32_t pwm)
{
    // 4096: ticks per turn
    float n = (float) nI * 1000 * 60 / 4096; //ticks/ms -> turns/min
    float T_f = 1000.0 / 406.0; //übersetzung getriebe + Nm->m
    float motor_a = 265.0; //Drehzahlkonstante   [min-1 V-1]
    float motor_b = 620.0;   //Kennliniensteigung [min-1 mNm-1]
    float motor_eta = 0.55 * 0.78; //Wirkungsgrad Getriebe *Motor, max from datasheet:0.75*0.85
    float Umax = 48.0; //Spannung bei pwm max
    float pwmmax = 3000.0;
    float pwm_zero = 400.0;
    float U;
    float T_motor;


    U = (float(std::fabs(pwm) - pwm_zero) / (pwmmax - pwm_zero))  *   Umax;
    if (U < 0)
    {
        U = 0;
    }
    if (pwm < 0)
    {
        U *= -1;
    }
    if (pwm == 0)
    {
        U = 0;
    }

    //U(M,n)=(n-b*M)/a
    T_motor = (U * motor_a - n) / -motor_b;
    auto T = T_motor * motor_eta / T_f;

    return T;
}

/* --------- !!! the following function are not used anymore, instead use the neural network for "motor22" !!! ----------- */
float MotorTorqueEstimator::linearModel_dcx22(float nI, int32_t pwm)
{
    float n = nI * 60.f / M_PI / 2.f * 231.f; //rad/s -> turns/min
    float T_f = 1000.f / 231.f; //übersetzung getriebe + Nm->m
    float motor_a = 226.f - 60.f; //Drehzahlkonstante   [min-1 V-1]
    float motor_b = 123.f;   //Kennliniensteigung [min-1 mNm-1]
    float motor_eta = 0.8f; //Wirkungsgrad Getriebe *Motor, max from datasheet:0.75*0.85
    float Umax = 48.f; //Spannung bei pwm max
    float Ulost = 0.f;
    float pwmmax = 3000.f;
    //float pwm_zero = 0.f;

    //float U = std::max(0.f, (abs(static_cast<float>(pwm)) - pwm_zero) / (pwmmax - pwm_zero) * (Umax - Ulost)) * (pwm >= 0 ? 1.f : -1.f);
    float U = (pwm / pwmmax) * (Umax - Ulost);
    float T_motor = (U * motor_a - n) / -motor_b;
    return (T_motor * motor_eta / T_f);
}

void MotorTorqueEstimator::estimateTorque(float vel, int32_t pwm)//, int motorModel, bool linearModelOnly = false)
{
    torque = 0.0f;
    // calculate the torque induced by non-linear model
    if (!linearModelOnly)
    {
        float vel_input = (float)vel / vel_factor;
        float pwm_input = (float)pwm / pwm_factor;
        //        static float pwmXvel_old = 0;
        float pwmXvel = vel_input * pwm_input;
        //        if (pwmXvel < 0)
        //        {
        //            pwmXvel = -1;
        //            pwmXvel_old = pwmXvel;
        //        }
        //        else if (pwmXvel > 0)
        //        {
        //            pwmXvel = 1;
        //            pwmXvel_old = pwmXvel;
        //        }
        //        else
        //        {
        //            pwmXvel = pwmXvel_old;
        //        }

        torque += network->feedforward(vel_input, pwm_input, pwmXvel);
        //        torque += network->feedforward(vel_input, pwm_input, 0.0f);
    }

    /*// calculating the torque induced by motor inertia
    double timeNow = IceUtil::Time::now().toMilliSecondsDouble();
    //    float deltaT = timeNow - last_time;
    float deltaT = 1; // in millisecond
    last_time = timeNow;

    float angVel = (float) 2 * M_PI * vel / 4.096 / deltaT;  //radian / s
    angVel = lowpassFilterFactor * angVel + (1 - lowpassFilterFactor) * vel_old;
    float acc = (float) 0.001 * (angVel - vel_old) / deltaT;
    vel_old = angVel;
    float inertiaTorque = acc * inertia;*/

    /* --------- !!! Do not change the following if-code for motor model "motor16" !!! ----------- */
    if (motorModel == "Motor16")
    {
        //        ARMARX_INFO << deactivateSpam(0.01) << "motor 16, torque: " << torque;
        torque = linearModel_dcx16(vel, pwm);
    }

    torqueWithInertia = torque;// + inertiaTorque;
    //    ARMARX_INFO << deactivateSpam(0.01) << "motor 16, torqueWithInertia: " << torqueWithInertia;
}
