#pragma once

#include <memory>

#include "TorqueEstimationNetwork.h"
#include "TorqueEstimationNetworkWeights.h"

class MotorTorqueEstimator
{
public:
    MotorTorqueEstimator(size_t index, bool nonlinearModelEnabled = false, std::string motorType = "None", float motorInertia = 0.0)
        : actorIndex(index), linearModelOnly(!nonlinearModelEnabled), motorModel(motorType), inertia(motorInertia),
          torque(0.0f), torqueWithInertia(0.0f), inertiaEnabled(true), vel_old(0.0f), last_time(0.0f), lowpassFilterFactor(0.7f)
    {
        network.reset(new TorqueEstimationNetwork);
    }
    ~MotorTorqueEstimator() {}

    void estimateTorque(float n, int32_t pwm);

    size_t getActorIndex()
    {
        return actorIndex;
    }
    float getTorque()
    {
        return torque;
    }
    float getTorqueWithInertia()
    {
        return torqueWithInertia;
    }

private:
    size_t actorIndex;
    bool linearModelOnly;
    std::string motorModel;
    float inertia;

    float torque;
    float torqueWithInertia;
    bool inertiaEnabled;
    float vel_old;
    double last_time;
    float lowpassFilterFactor;

    float linearModel_dcx16(int32_t nI, int32_t pwm);
    float linearModel_dcx22(float nI, int32_t pwm);

    TorqueEstimationNetworkPtr network;
};
using MotorTorqueEstimatorPtr = std::shared_ptr<MotorTorqueEstimator>;
