#include "TorqueEstimationNetwork.h"
#include "TorqueEstimationNetworkWeights.h"

#include <algorithm>

template <unsigned int layer_in_dim, unsigned int layer_out_dim>
void TorqueEstimationNetwork::fully_connected_layer_sigmoid_forward(float (&layer_in) [layer_in_dim],
        float (&layer_out) [layer_out_dim],
        const float (&layer_weights) [layer_in_dim][layer_out_dim],
        const float (&layer_bias) [layer_out_dim])
{
    std::fill_n(layer_out, layer_out_dim, 0.0f);
    for (unsigned int outF = 0; outF < layer_out_dim; outF++)
    {
        for (unsigned int inF = 0; inF < layer_in_dim; inF++)
        {
            layer_out[outF] += layer_in[inF] * layer_weights[inF][outF];
        }

        //BIAS
        layer_out[outF] += layer_bias[outF];

        //SIGMOID
        layer_out[outF] = std::max(0.0f, std::min(1.0f, layer_out[outF] * 0.2f + 0.5f));
    }
}

template <unsigned int layer_in_dim, unsigned int layer_out_dim>
void TorqueEstimationNetwork::fully_connected_layer_forward(float (&layer_in) [layer_in_dim],
        float (&layer_out) [layer_out_dim],
        const float (&layer_weights) [layer_in_dim][layer_out_dim],
        const float (&layer_bias) [layer_out_dim])
{
    std::fill_n(layer_out, layer_out_dim, 0.0f);
    for (unsigned int outF = 0; outF < layer_out_dim; outF++)
    {
        for (unsigned int inF = 0; inF < layer_in_dim; inF++)
        {
            layer_out[outF] += layer_in[inF] * layer_weights[inF][outF];
        }

        //BIAS
        layer_out[outF] += layer_bias[outF];
    }
}

float TorqueEstimationNetwork::feedforward(float vel_input, float pwm_input, float pwmXvel)
{
    layer_in[0] = vel_input;
    layer_in[1] = pwm_input;
    layer_in[2] = vel_input;
    layer_in[3] = pwm_input;
    layer_in[4] = pwmXvel;

    fully_connected_layer_sigmoid_forward(layer_in, fcl1, fc1Weights, fc1Bias);
    fully_connected_layer_sigmoid_forward(fcl1, fcl2, fc2Weights, fc2Bias);
    fully_connected_layer_forward(fcl2, fcl3, fc3Weights, fc3Bias);

    return fcl3[0];
}
