import h5py


def traverse_datasets(hdf_file):
    def h5py_dataset_iterator(g, prefix=''):
        for key in g.keys():
            item = g[key]
            path = f'{prefix}/{key}'
            if isinstance(item, h5py.Dataset):  # test for dataset
                yield (path, item)
            elif isinstance(item, h5py.Group):  # test for group (go down)
                yield from h5py_dataset_iterator(item, path)

    with h5py.File(hdf_file, 'r') as f:
        for path, _ in h5py_dataset_iterator(f):
            yield path

def oned1array(array):
    return f'{{{"f, ".join(str(x) for x in array)}}}'

def run(filename):
    with h5py.File(filename, 'r') as f:
        print("namespace torqueEstimation")
        print("{")

        for dset in traverse_datasets(filename):
            #print('Path:', dset)
            #print('Shape:', f[dset].shape)
            #print('Dimension:', len(f[dset].shape))
            #print('Data type:', f[dset].dtype)

            if len(f[dset].shape) == 1:
                print(f'\tconst float {dset}[{f[dset].shape[0]}] =')
                print(f'\t\t{oned1array(f[dset])};')
                print("\n")
            if len(f[dset].shape) == 2:
                print(f'\tconst float {dset}[{f[dset].shape[0]}][{f[dset].shape[1]}] =')
                print("\t{")
                arrays = []
                for array in f[dset]:
                    arrays.append(oned1array(array))
                limiter = ", \n\t\t"
                print(f'\t\t{limiter.join(str(arr) for arr in arrays)}')
                print("\t};")
                print("\n")

        print("}")

if __name__ == "__main__":
    run("NN_T_predict_nonneg_hard_sigmoid_16.h5")
