#include "NJointGripperZeroTorqueController.h"
#include <VirtualRobot/RobotNodeSet.h>

namespace armarx
{
    NJointControllerRegistration<NJointGripperZeroTorqueController> registrationControllerNJointGripperZeroTorqueController("NJointGripperZeroTorqueController");

    NJointGripperZeroTorqueController::NJointGripperZeroTorqueController(const RobotUnitPtr&, const armarx::NJointControllerConfigPtr& config, const VirtualRobot::RobotPtr&)
    {
        useSynchronizedRtRobot();
        NJointGripperZeroTorqueControllerConfigPtr cfg = NJointGripperZeroTorqueControllerConfigPtr::dynamicCast(config);
        VirtualRobot::RobotNodeSetPtr rns = rtGetRobot()->getRobotNodeSet(cfg->nodeSetName);

        for (size_t i = 0; i < rns->getSize(); ++i)
        {
            std::string jointName = rns->getNode(i)->getName();
            ControlTargetBase* ct = useControlTarget(jointName, ControlModes::ZeroTorque1DoF);
            targets.push_back(ct->asA<ControlTarget1DoFActuatorZeroTorque>());
        };
        //        ControlTargetBase* ctPump = useControlTarget("KIT_Gripper_Pump", ControlModes::Velocity1DoF);
        //        pumpTarget.push_back(ctPump->asA<ControlTarget1DoFActuatorVelocity>());
    }

    std::string NJointGripperZeroTorqueController::getClassName(const Ice::Current&) const
    {
        return "NJointGripperZeroTorqueController";
    }

    void NJointGripperZeroTorqueController::rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration)
    {
        for (size_t i = 0; i < targets.size(); ++i)
        {
            targets.at(i)->torque = 0;
        }

        //        pumpTarget.at(0)->velocity = 70.0;
    }

    WidgetDescription::WidgetPtr NJointGripperZeroTorqueController::GenerateConfigDescription(const VirtualRobot::RobotPtr& robot, const std::map<std::string, ConstControlDevicePtr>& controlDevices, const std::map<std::string, ConstSensorDevicePtr>&)
    {
        using namespace armarx::WidgetDescription;
        HBoxLayoutPtr layout = new HBoxLayout;

        LabelPtr label = new Label;
        label->text = "nodeset name";
        layout->children.emplace_back(label);

        StringComboBoxPtr box = new StringComboBox;
        box->defaultIndex = 0;
        box->options = robot->getRobotNodeSetNames();
        box->name = "nodeSetName";
        layout->children.emplace_back(box);

        return layout;
    }

    NJointGripperZeroTorqueControllerConfigPtr NJointGripperZeroTorqueController::GenerateConfigFromVariants(const StringVariantBaseMap& values)
    {
        return new NJointGripperZeroTorqueControllerConfig(values.at("nodeSetName")->getString());
    }
}
